@@ -2,6 +2,14 @@ package Jifty::DBI::Collection;
 
 use warnings;
 use strict;
+use Scalar::Defer qw/lazy/;
+use Scalar::Util qw/weaken/;
+use overload (
+    '@{}'       => \&items_array_ref,
+    '<>'        => \&next,
+    bool        => sub {shift},
+    fallback    => 1
+);
 
 =head1 NAME
 
@@ -60,7 +68,7 @@ use Data::Page;
 use Clone;
 use Carp qw/croak/;
 use base qw/Class::Accessor::Fast/;
-__PACKAGE__->mk_accessors(qw/pager prefetch_related derived/);
+__PACKAGE__->mk_accessors(qw/pager prefetch_related derived _handle _is_limited rows_per_page/);
 
 =head1 METHODS
 
@@ -115,11 +123,7 @@ sub _init {
 
 sub _init_pager {
     my $self = shift;
-    $self->pager( Data::Page->new );
-
-    $self->pager->total_entries(0);
-    $self->pager->entries_per_page(10);
-    $self->pager->current_page(1);
+    return $self->pager( Data::Page->new(0, 10, 1) );
 }
 
 =head2 clean_slate
@@ -144,7 +148,6 @@ sub clean_slate {
     $self->{'order'}            = "";
     $self->{'alias_count'}      = 0;
     $self->{'first_row'}        = 0;
-    $self->{'show_rows'}        = 0;
 
     delete $self->{$_} for qw(
         items
@@ -157,6 +160,7 @@ sub clean_slate {
         criteria_count
     );
 
+    $self->rows_per_page(0);
     $self->implicit_clauses(%args);
     $self->_is_limited(0);
 }
@@ -177,14 +181,6 @@ Get or set this object's L<Jifty::DBI::Handle> object.
 
 =cut
 
-sub _handle {
-    my $self = shift;
-    if (@_) {
-        $self->{'DBIxhandle'} = shift;
-    }
-    return ( $self->{'DBIxhandle'} );
-}
-
 =head2 _do_search
 
 This internal private method actually executes the search on the
@@ -206,25 +202,39 @@ sub _do_search {
     my @names = @{ $records->{NAME_lc} };
     my $data  = {};
 
-    my @tables = (
-        "main", map { $_->{alias} } values %{ $self->prefetch_related || {} }
-    );
+    my @tables = map { $_->{alias} } values %{ $self->prefetch_related || {} };
+
+    unless ( @tables ) {
+        while ( my $row = $records->fetchrow_hashref() ) {
+            $row->{ substr($_, 5) } = delete $row->{ $_ }
+                foreach grep rindex($_, "main_", 0) == 0, keys %$row;
+            my $item = $self->new_item;
+            $item->load_from_hash($row, fast => 1);
+            $self->add_record($item);
+        }
+        if ( $records->err ) {
+            $self->{'must_redo_search'} = 0;
+        }
+
+        return $self->_record_count;
+    }
 
     my @order;
+    my $i = 1;
     while ( my $base_row = $records->fetchrow_hashref() ) {
         my $main_pkey = $base_row->{ $names[0] };
+        $main_pkey = 'unique-'.$i++ if $self->{group_by};
         push @order, $main_pkey
             unless ( $order[0] && $order[-1] eq $main_pkey );
 
         # let's chop the row into subrows;
-        foreach my $table (@tables) {
-            for ( keys %$base_row ) {
-                if (/^$table\_(.*)$/) {
-                    $data->{$main_pkey}->{$table}
-                        ->{ ( $base_row->{ $table . '_id' } || $main_pkey ) }
-                        ->{$1} = $base_row->{$_};
-                }
+        foreach my $table ('main', @tables) {
+            my %tmp = ();
+            for my $k( grep rindex($_, $table ."_", 0) == 0, keys %$base_row ) {
+                $tmp{ substr($k, length($table)+1) } = $base_row->{ $k };
             }
+            $data->{$main_pkey}{$table}{ $base_row->{ $table . '_id' } || $main_pkey }
+                = \%tmp if keys %tmp;
         }
     }
 
@@ -232,7 +242,7 @@ sub _do_search {
         my $item;
         foreach my $row ( values %{ $data->{$row_id}->{'main'} } ) {
             $item = $self->new_item();
-            $item->load_from_hash($row);
+            $item->load_from_hash($row, fast => 1);
         }
         foreach my $alias ( grep { $_ ne 'main' } keys %{ $data->{$row_id} } )
         {
@@ -250,7 +260,7 @@ sub _do_search {
                     derived => 1 );
                 foreach my $row (@rows) {
                     my $entry = $collection->new_item;
-                    $entry->load_from_hash($row);
+                    $entry->load_from_hash($row, fast => 1);
                     $collection->add_record($entry);
                 }
 
@@ -259,7 +269,7 @@ sub _do_search {
                 warn "Multiple rows returned for $class in prefetch"
                     if @rows > 1;
                 my $entry = $class->new( $self->_new_record_args );
-                $entry->load_from_hash( shift @rows ) if @rows;
+                $entry->load_from_hash( shift(@rows), fast => 1 ) if @rows;
                 $item->prefetched( $col_name => $entry );
             } else {
                 Carp::cluck(
@@ -349,7 +359,7 @@ only C<< $self->rows_per_page >> rows, skipping C<< $self->first_row >>
 rows.  (That is, if rows are numbered starting from 0, row number
 C<< $self->first_row >> will be the first row returned.)  Note that it
 probably makes no sense to set these variables unless you are also
-enforcing an ordering on the rows (with L</order_by_cols>, say).
+enforcing an ordering on the rows (with L</order_by>, say).
 
 =cut
 
@@ -453,15 +463,6 @@ C<0> means "no limits have been applied yet.
 
 =cut
 
-sub _is_limited {
-    my $self = shift;
-    if (@_) {
-        $self->{'is_limited'} = shift;
-    } else {
-        return ( $self->{'is_limited'} );
-    }
-}
-
 =head2 build_select_query
 
 Builds a query string for a "SELECT rows from Tables" statement for
@@ -509,11 +510,10 @@ sub query_columns {
     my $self = shift;
 
     my @cols = ();
-    my $item = $self->new_item;
     if ( $self->{columns} and @{ $self->{columns} } ) {
         push @cols, @{ $self->{columns} };
     } else {
-        push @cols, $self->_qualified_record_columns( 'main' => $item );
+        push @cols, $self->_qualified_record_columns( 'main' => $self->new_item );
     }
     my %prefetch_related = %{ $self->prefetch_related || {} };
     foreach my $alias ( keys %prefetch_related ) {
@@ -521,7 +521,7 @@ sub query_columns {
 
         my $reference;
         if ( $class->isa('Jifty::DBI::Collection') ) {
-            $reference = $class->new->new_item( $self->_new_collection_args );
+            $reference = $class->new( $self->_new_collection_args )->new_item;
         } elsif ( $class->isa('Jifty::DBI::Record') ) {
             $reference = $class->new( $self->_new_record_args );
         }
@@ -553,15 +553,8 @@ sub _qualified_record_columns {
     my $self  = shift;
     my $alias = shift;
     my $item  = shift;
-    grep {$_} map {
-        my $col = $_;
-        if ( $col->virtual ) {
-            undef;
-        } else {
-            $col = $col->name;
-            $alias . "." . $col . " as " . $alias . "_" . $col;
-        }
-    } $item->columns;
+    return map $alias ."." . $_ ." as ". $alias ."_". $_,
+        map $_->name, grep !$_->virtual, $item->columns;
 }
 
 =head2 prefetch PARAMHASH
@@ -762,11 +755,11 @@ sub resolve_join {
                 = $classname->new( $self->_new_collection_args )->new_item;
             my $right_alias = $self->new_alias($item);
             $self->join(
-                type    => 'left',
-                alias1  => $last_alias,
-                column1 => 'id',
-                alias2  => $right_alias,
-                column2 => $column->by || 'id',
+                type        => 'left',
+                alias1      => $last_alias,
+                column1     => 'id',
+                alias2      => $right_alias,
+                column2     => $column->by || 'id',
                 is_distinct => 1,
             );
             $last_alias = $right_alias;
@@ -774,11 +767,11 @@ sub resolve_join {
             my $item        = $classname->new( $self->_new_record_args );
             my $right_alias = $self->new_alias($item);
             $self->join(
-                type    => 'left',
-                alias1  => $last_alias,
-                column1 => $name,
-                alias2  => $right_alias,
-                column2 => $column->by || 'id',
+                type        => 'left',
+                alias1      => $last_alias,
+                column1     => $name,
+                alias2      => $right_alias,
+                column2     => $column->by || 'id',
                 is_distinct => 1,
             );
             $last_alias = $right_alias;
@@ -851,7 +844,7 @@ on your database, call C<do_search> before that C<count>.
 
 sub do_search {
     my $self = shift;
-    return              if $self->derived;
+    return if $self->derived;
     $self->_do_search() if $self->{'must_redo_search'};
 
 }
@@ -864,6 +857,13 @@ returns undef and resets the search such that the following call to
 L</next> will start over with the first item retrieved from the
 database.
 
+You may also call this method via the built-in iterator syntax.
+The two lines below are equivalent:
+
+    while ($_ = $collection->next) { ... }
+
+    while (<$collection>) { ... }
+
 =cut
 
 sub next {
@@ -957,6 +957,13 @@ sub last {
 Return a reference to an array containing all objects found by this
 search.
 
+You may also call this method via the built-in array dereference syntax.
+The two lines below are equivalent:
+
+    for (@{$collection->items_array_ref}) { ... }
+
+    for (@$collection) { ... }
+
 =cut
 
 sub items_array_ref {
@@ -976,9 +983,16 @@ sub items_array_ref {
 =head2 new_item
 
 Should return a new object of the correct type for the current collection.
+L</record_class> method is used to determine class of the object.
+
+Each record class at least once is loaded using require. This method is
+called each time a record fetched so load atemts are cached to avoid
+penalties. If you're sure that all record classes are loaded before
+first use then you can override this method.
 
 =cut
 
+{ my %cache = ();
 sub new_item {
     my $self  = shift;
     my $class = $self->record_class();
@@ -986,15 +1000,17 @@ sub new_item {
     die "Jifty::DBI::Collection needs to be subclassed; override new_item\n"
         unless $class;
 
-    warn "$self $class\n" if $class =~ /::$/;
-    $class->require();
+    unless ( exists $cache{$class} ) {
+        $class->require;
+        $cache{$class} = undef;
+    }
     return $class->new( $self->_new_record_args );
-}
+} }
 
 =head2 record_class
 
 Returns the record class which this is a collection of; override this
-to subclass.  Or, pass it the name of a class an an argument after
+to subclass.  Or, pass it the name of a class as an argument after
 creating a C<Jifty::DBI::Collection> object to create an 'anonymous'
 collection class.
 
@@ -1002,9 +1018,9 @@ If you haven't specified a record class, this returns a best guess at
 the name of the record class for this collection.
 
 It uses a simple heuristic to determine the record class name -- It
-chops "Collection" off its own name. If you want to name your records
-and collections differently, go right ahead, but don't say we didn't
-warn you.
+chops "Collection" or "s" off its own name. If you want to name your
+records and collections differently, go right ahead, but don't say we
+didn't warn you.
 
 =cut
 
@@ -1016,7 +1032,7 @@ sub record_class {
             if ref $self->{record_class};
     } elsif ( not ref $self or not $self->{record_class} ) {
         my $class = ref($self) || $self;
-        $class =~ s/(Collection|s)$//
+        $class =~ s/(?<!:)(Collection|s)$//
             || die "Can't guess record class from $class";
         return $class unless ref $self;
         $self->{record_class} = $class;
@@ -1115,11 +1131,11 @@ database supports are also valid.
 
 MATCHES is like LIKE, except it surrounds the value with % signs.
 
-=item "STARTSWITH"
+=item "starts_with"
 
-STARTSWITH is like LIKE, except it only appends a % at the end of the string
+starts_with is like LIKE, except it only appends a % at the end of the string
 
-=item "ENDSWITH"
+=item "ends_with"
 
 ENDSWITH is like LIKE, except it prepends a % to the beginning of the string
 
@@ -1128,6 +1144,12 @@ ENDSWITH is like LIKE, except it prepends a % to the beginning of the string
 IN matches a column within a set of values.  The value specified in the limit
 should be an array reference of values.
 
+=item "IS"
+
+=item "IS NOT"
+
+This is useful for when you wish to match columns that contain NULL (or ones that don't). Use this operator and a value of "NULL".
+
 =back
 
 =item escape
@@ -1153,10 +1175,10 @@ is numeric.
 sub limit {
     my $self = shift;
     my %args = (
-        table            => $self->table,
+        table            => undef,
+        alias            => undef,
         column           => undef,
         value            => undef,
-        alias            => undef,
         quote_value      => 1,
         entry_aggregator => 'or',
         case_sensitive   => undef,
@@ -1169,6 +1191,59 @@ sub limit {
 
     return if $self->derived;
 
+    #If we're performing a left join, we really want the alias to be the
+    #left join criterion.
+
+    if (   ( defined $args{'leftjoin'} )
+        && ( not defined $args{'alias'} ) )
+    {
+        $args{'alias'} = $args{'leftjoin'};
+    }
+
+    # {{{ if there's no alias set, we need to set it
+
+    unless ( defined $args{'alias'} ) {
+
+        #if the table we're looking at is the same as the main table
+        if ( !defined $args{'table'} || $args{'table'} eq $self->table ) {
+
+            # TODO this code assumes no self joins on that table.
+            # if someone can name a case where we'd want to do that,
+            # I'll change it.
+
+            $args{'alias'} = 'main';
+        }
+
+        else {
+            $args{'alias'} = $self->new_alias( $args{'table'} );
+        }
+    }
+
+    # }}}
+
+    # Set this to the name of the column and the alias, unless we've been
+    # handed a subclause name
+
+    my $qualified_column
+        = $args{'alias'}
+        ? $args{'alias'} . "." . $args{'column'}
+        : $args{'column'};
+    my $clause_id = $args{'subclause'} || $qualified_column;
+
+    # XXX: when is column_obj undefined?
+    my $class
+        = $self->{joins}{ $args{alias} }
+        && $self->{joins}{ $args{alias} }{class}
+        ? $self->{joins}{ $args{alias} }{class}
+        ->new( $self->_new_collection_args )
+        : $self;
+    my $column_obj = $class->new_item()->column( $args{column} );
+
+    $self->record_class->new(handle => $self->_handle)->_apply_input_filters(
+        column    => $column_obj,
+        value_ref => \$args{'value'},
+    ) if $column_obj && $column_obj->encode_on_select;
+
     # make passing in an object DTRT
     my $value_ref = ref( $args{value} );
     if ($value_ref) {
@@ -1198,12 +1273,12 @@ sub limit {
         #If it's a like, we supply the %s around the search term
         if ( $args{'operator'} =~ /MATCHES/i ) {
             $args{'value'} = "%" . $args{'value'} . "%";
-        } elsif ( $args{'operator'} =~ /STARTSWITH/i ) {
+        } elsif ( $args{'operator'} =~ /STARTS_?WITH/i ) {
             $args{'value'} = $args{'value'} . "%";
-        } elsif ( $args{'operator'} =~ /ENDSWITH/i ) {
+        } elsif ( $args{'operator'} =~ /ENDS_?WITH/i ) {
             $args{'value'} = "%" . $args{'value'};
         }
-        $args{'operator'} =~ s/(?:MATCHES|ENDSWITH|STARTSWITH)/LIKE/i;
+        $args{'operator'} =~ s/(?:MATCHES|ENDS_?WITH|STARTS_?WITH)/LIKE/i;
 
         #if we're explicitly told not to to quote the value or
         # we're doing an IS or IS NOT (null), don't quote the operator.
@@ -1221,45 +1296,6 @@ sub limit {
         $args{'escape'} = 'ESCAPE ' . $self->_quote_value( $args{escape} );
     }
 
-    #If we're performing a left join, we really want the alias to be the
-    #left join criterion.
-
-    if (   ( defined $args{'leftjoin'} )
-        && ( not defined $args{'alias'} ) )
-    {
-        $args{'alias'} = $args{'leftjoin'};
-    }
-
-    # {{{ if there's no alias set, we need to set it
-
-    unless ( defined $args{'alias'} ) {
-
-        #if the table we're looking at is the same as the main table
-        if ( $args{'table'} eq $self->table ) {
-
-            # TODO this code assumes no self joins on that table.
-            # if someone can name a case where we'd want to do that,
-            # I'll change it.
-
-            $args{'alias'} = 'main';
-        }
-
-        else {
-            $args{'alias'} = $self->new_alias( $args{'table'} );
-        }
-    }
-
-    # }}}
-
-    # Set this to the name of the column and the alias, unless we've been
-    # handed a subclause name
-
-    my $qualified_column
-        = $args{'alias'}
-        ? $args{'alias'} . "." . $args{'column'}
-        : $args{'column'};
-    my $clause_id = $args{'subclause'} || $qualified_column;
-
     # If we're trying to get a leftjoin restriction, lets set
     # $restriction to point there. otherwise, lets construct normally
 
@@ -1274,22 +1310,21 @@ sub limit {
 
     # If it's a new value or we're overwriting this sort of restriction,
 
-    # XXX: when is column_obj undefined?
-    my $class = $self->{joins}{$args{alias}} && $self->{joins}{$args{alias}}{class} 
-      ? $self->{joins}{$args{alias}}{class}->new($self->_new_collection_args)
-      : $self;
-    my $column_obj = $class->new_item()->column( $args{column} );
-    my $case_sensitive = $column_obj ? $column_obj->case_sensitive : 0;
-    $case_sensitive = $args{'case_sensitive'}
-        if defined $args{'case_sensitive'};
-    if (   $self->_handle->case_sensitive
-        && defined $args{'value'}
-        && $args{'quote_value'}
-        && !$case_sensitive )
-    {
-
+    if ( defined $args{'value'} && $args{'quote_value'} ) {
+        my $case_sensitive = 0;
+        if ( defined $args{'case_sensitive'} ) {
+            $case_sensitive = $args{'case_sensitive'};
+        }
+        elsif ( $column_obj ) {
+            $case_sensitive = $column_obj->case_sensitive;
+        }
         # don't worry about case for numeric columns_in_db
-        if ( defined $column_obj ? $column_obj->is_string : 1 ) {
+        # only be case insensitive when we KNOW it's a text
+        if ( $column_obj && !$case_sensitive && !$column_obj->is_string ) {
+            $case_sensitive = 1;
+        }
+
+        if ( !$case_sensitive && $self->_handle->case_sensitive ) {
             ( $qualified_column, $args{'operator'}, $args{'value'} )
                 = $self->_handle->_make_clause_case_insensitive(
                 $qualified_column, $args{'operator'}, $args{'value'} );
@@ -1342,6 +1377,35 @@ of allowing you to construct malformed SQL queries.  Its interface
 will probably change in the near future, but its presence allows for
 arbitrarily complex queries.
 
+Here's an example, to construct a SQL WHERE clause roughly equivalent to (depending on your SQL dialect):
+
+  parent = 12 AND task_type = 'action' 
+      AND (status = 'open' 
+          OR (status = 'done' 
+              AND completed_on >= '2008-06-26 11:39:22'))
+
+You can use sub-clauses and C<open_paren> and C<close_paren> as follows:
+
+  $col->limit( column => 'parent', value => 12 );
+  $col->limit( column => 'task_type', value => 'action' );
+
+  $col->open_paren("my_clause");
+
+  $col->limit( subclause => "my_clause", column => 'status', value => 'open' );
+
+  $col->open_paren("my_clause");
+
+  $col->limit( subclause => "my_clause", column => 'status', 
+      value => 'done', entry_aggregator => 'OR' );
+  $col->limit( subclause => "my_clause", column => 'completed_on',
+      operator => '>=', value => '2008-06-26 11:39:22' );
+
+  $col->close_paren("my_clause");
+
+  $col->close_paren("my_clause");
+
+Where the C<"my_clause"> can be any name you choose.
+
 =cut
 
 sub open_paren {
@@ -1416,7 +1480,9 @@ sub _compile_generic_restrictions {
             unless ( ref $entry ) {
                 $result .= ' ' . $entry . ' ';
             } else {
-                $result .= join ' ', grep { defined } @{$entry}{qw(column operator value escape)};
+                $result .= join ' ',
+                    grep {defined}
+                    @{$entry}{qw(column operator value escape)};
             }
         }
         $result .= ')';
@@ -1483,6 +1549,9 @@ in the C<alias.column> format.
 
 Use array of hashes to order by many columns/functions.
 
+Calling this I<sets> the ordering, it doesn't refine it. If you want to keep
+previous ordering, use C<add_order_by>.
+
 The results would be unordered if method called without arguments.
 
 Returns the current list of columns.
@@ -1490,6 +1559,22 @@ Returns the current list of columns.
 =cut
 
 sub order_by {
+    my $self = shift;
+    return if $self->derived;
+    if (@_) {
+        $self->{'order_by'} = [];
+        $self->add_order_by(@_);
+    }
+    return ( $self->{'order_by'} || [] );
+}
+
+=head2 add_order_by EMPTY|HASH|ARRAY_OF_HASHES
+
+Same as order_by, except it will not reset the ordering you have already set.
+
+=cut
+
+sub add_order_by {
     my $self = shift;
     return if $self->derived;
     if (@_) {
@@ -1498,12 +1583,24 @@ sub order_by {
         unless ( UNIVERSAL::isa( $args[0], 'HASH' ) ) {
             @args = {@args};
         }
-        $self->{'order_by'} = \@args;
+        push @{ $self->{'order_by'} ||= [] }, @args;
         $self->redo_search();
     }
     return ( $self->{'order_by'} || [] );
 }
 
+=head2 clear_order_by
+
+Clears whatever would normally get set in the ORDER BY clause.
+
+=cut
+
+sub clear_order_by {
+    my $self = shift;
+
+    $self->{'order_by'} = [];
+}
+
 =head2 _order_clause
 
 returns the ORDER BY clause for the search.
@@ -1646,9 +1743,10 @@ sub new_alias {
     my $self = shift;
     my $refers_to = shift || die "Missing parameter";
     my $table;
-
+    my $class = undef;
     if ( $refers_to->can('table') ) {
         $table = $refers_to->table;
+        $class = $refers_to;
     } else {
         $table = $refers_to;
     }
@@ -1656,9 +1754,10 @@ sub new_alias {
     my $alias = $self->_get_alias($table);
 
     $self->{'joins'}{$alias} = {
-        alias        => $alias,
-        table        => $table,
-        type         => 'CROSS',
+        alias => $alias,
+        table => $table,
+        type  => 'CROSS',
+        ( $class ? ( class => $class ) : () ),
         alias_string => " CROSS JOIN $table $alias ",
     };
 
@@ -1674,11 +1773,7 @@ sub _get_alias {
     my $self  = shift;
     my $table = shift;
 
-    $self->{'alias_count'}++;
-    my $alias = $table . "_" . $self->{'alias_count'};
-
-    return ($alias);
-
+    return $table . "_" . ++$self->{'alias_count'};
 }
 
 =head2 join
@@ -1740,18 +1835,24 @@ the selected page.
 sub set_page_info {
     my $self = shift;
     my %args = (
-        per_page     => undef,
-        current_page => undef,    # 1-based
+        per_page     => 0,
+        current_page => 1,    # 1-based
         @_
     );
     return if $self->derived;
 
-    $self->pager->total_entries( $self->count_all )
+    my $weakself = $self;
+    weaken($weakself);
+
+    $self->pager->total_entries( lazy { $weakself->count_all } )
         ->entries_per_page( $args{'per_page'} )
         ->current_page( $args{'current_page'} );
 
     $self->rows_per_page( $args{'per_page'} );
-    $self->first_row( $self->pager->first || 1 );
+
+    # We're not using $pager->first because it automatically does a count_all
+    # to correctly return '0' for empty collections
+    $self->first_row( ( $args{'current_page'} - 1 ) * $args{'per_page'} + 1 );
 
 }
 
@@ -1763,13 +1864,6 @@ the number of rows the database should display.
 
 =cut
 
-sub rows_per_page {
-    my $self = shift;
-    $self->{'show_rows'} = shift if (@_);
-
-    return ( $self->{'show_rows'} );
-}
-
 =head2 first_row
 
 Get or set the first row of the result set the database should return.
@@ -2001,7 +2095,8 @@ sub column {
 
 =head2 columns LIST
 
-Specify that we want to load only the columns in LIST, which is a 
+Specify that we want to load only the columns in LIST, which should be
+a list of column names.
 
 =cut
 
