@@ -37,6 +37,7 @@ my @attrs = qw/
 my @handy_attrs = qw/
     container
     label hints render_as
+    display_length
     documentation
     valid_values
     available_values
@@ -141,6 +142,8 @@ known attributes are:
 
 =item     label hints render_as
 
+=item     display_length
+
 =item     valid_values
 
 =item     available_values
