@@ -175,7 +175,7 @@ sub _make_clause_case_insensitive {
     if ( $self->_case_insensitivity_valid( $column, $operator, $value ) ) {
         if ( $operator =~ /(?:LIKE|=|IN)/i ) {
             $column = "LOWER($column)";
-            if ( $operator eq 'IN' and ref($value) eq 'ARRAY' ) {
+            if ( $operator =~ /^(IN|=)$/i and ref($value) eq 'ARRAY' ) {
                 $value = [ map {"LOWER($_)"} @$value ];
             } else {
                 $value = "LOWER($value)";
@@ -236,6 +236,22 @@ sub distinct_query {
     }
 }
 
+=head2 canonical_true
+
+The canonical true value in Postgres is 't'.
+
+=cut
+
+sub canonical_true { 't' }
+
+=head2 canonical_false
+
+The canonical false value in Postgres is 'f'.
+
+=cut
+
+sub canonical_false { 'f' }
+
 1;
 
 __END__
