@@ -34,7 +34,7 @@ that user has installed.
 
 our @available_drivers = grep { eval "require DBD::". $_ } @supported_drivers;
 
-=head1 functionS
+=head1 FUNCTIONS
 
 =head2 get_handle
 
@@ -288,6 +288,24 @@ sub cleanup_schema
 
 =head2 init_data
 
+Takes a class to get data from and the handle, calls C<init_data>
+method in the class, result is used to create new records of that
+class. First row is used for columns names.
+
+Example:
+
+    init_data('TestApp::User', $handle);
+
+    ...
+
+    package TestApp::User;
+    sub init_data { return (
+        ['name', 'email'],
+
+        ['ruz', 'ruz@localhost'],
+        ...
+    ) }
+
 =cut
 
 sub init_data
@@ -309,4 +327,27 @@ sub init_data
         return $count;
 }
 
+=head2 drop_table_if_exists
+
+Takes a table name and handle. Drops the table in the DB if it exists.
+Returns nothing interesting, shouldn't die.
+
+=cut
+
+sub drop_table_if_exists {
+    my ($table, $handle) = @_;
+    my $d = handle_to_driver( $handle );
+    if ( $d eq 'Pg' ) {
+        my ($exists) = $handle->dbh->selectrow_array(
+            "select 1 from pg_tables where tablename = ?", undef, $table
+        );
+        $handle->simple_query("DROP TABLE $table") if $exists;
+    }
+    else {
+        local $@;
+        eval { $handle->simple_query("DROP TABLE IF EXISTS $table") };
+    }
+    return;
+}
+
 1;
